/**
 * @author Kuitos
 * @homepage https://github.com/kuitos/
 * @since 2018-08-15 11:37
 */
import loadjs from 'loadjs';
import processTpl from './process-tpl';

function getDomain(url) {
  try {
    var href = new URL(url);
    return href.origin;
  } catch (e) {
    return '';
  }
}

function promisifySeriesLoadjs(scripts) {
  return new Promise(function (resolve, reject) {
    return scripts.length ? loadjs(scripts, {
      async: false,
      success: resolve,
      error: reject
    }) : resolve();
  });
}

var init = false;
var globalDefine = null;

function mountSystemJS() {
  if (!init) {
    // as systemjs will pollute the global variables, lazy mount systemjs
    require('systemjs/dist/s');

    require('systemjs/dist/extras/amd');

    require('systemjs/dist/extras/named-exports');

    init = true;
    globalDefine = window.define;
  } else {
    window.define = globalDefine;
  }
}

function unmountSystemJS() {
  delete window.define;
}

export default function importHTML(url, stripStyles) {
  var domain = getDomain(url);
  return fetch(url).then(function (response) {
    return response.text();
  }).then(function (html) {
    var _processTpl = processTpl(html, domain, stripStyles),
        template = _processTpl.template,
        scripts = _processTpl.scripts,
        entry = _processTpl.entry;

    return {
      template: template,
      // return the entry script exports
      loadScripts: function loadScripts() {
        var entryIndex = scripts.indexOf(entry);
        var preScripts = scripts.slice(0, entryIndex);
        var postScripts = scripts.slice(entryIndex + 1);
        var exports = null;
        return promisifySeriesLoadjs(preScripts).then(function () {
          mountSystemJS();
          exports = window.System.import(entry);
          return exports.then(unmountSystemJS);
        }).then(promisifySeriesLoadjs(postScripts)).then(function () {
          return exports;
        });
      }
    };
  });
}
;