/**
 * @author Kuitos
 * @homepage https://github.com/kuitos/
 * @since 2018-09-03 15:04
 */
var SCRIPT_TAG_REGEX = /<(script)\s+((?!type=('|')text\/ng-template\3).)*?>.*?<\/\1>/gi;
var SCRIPT_SRC_REGEX = /.*\ssrc=('|")(\S+)\1.*/;
var SCRIPT_ENTRY_REGEX = /.*\sentry\s*.*/;
var LINK_TAG_REGEX = /<(link)\s+.*?>/gi;
var STYLE_TYPE_REGEX = /\s+rel=("|')stylesheet\1.*/;
var STYLE_HREF_REGEX = /.*\shref=('|")(\S+)\1.*/;

function hasProtocol(url) {
  return url.startsWith('//') || url.startsWith('http');
}
/**
 * parse the script link from the template
 * TODO
 *    1. collect stylesheets
 *    2. use global eval to evaluate the inline scripts
 *        see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function#Difference_between_Function_constructor_and_function_declaration
 *        see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/eval#Do_not_ever_use_eval!
 * @param tpl
 * @param domain
 * @stripStyles whether to strip the css links
 * @returns {{template: void | string | *, scripts: *[], entry: *}}
 */


export default function processTpl(tpl, domain) {
  var stripStyles = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var scripts = [];
  var styles = [];
  var entry = null;
  var template = tpl.replace(LINK_TAG_REGEX, function (match) {
    /*
    change the css link
     */
    var styleType = !!match.match(STYLE_TYPE_REGEX);

    if (styleType) {
      var styleHref = match.match(STYLE_HREF_REGEX);

      if (styleHref) {
        var href = styleHref && styleHref[2];
        var newHref = href;

        if (href && !hasProtocol(href)) {
          newHref = domain + href;
        }

        if (stripStyles) {
          styles.push(newHref);
          return "<!-- link ".concat(newHref, " replaced -->");
        } else if (newHref !== href) {
          return match.replace(href, newHref);
        }
      }
    }

    return match;
  }).replace(SCRIPT_TAG_REGEX, function (match) {
    /*
    collect scripts and replace the ref
     */
    var matchedScriptEntry = match.match(SCRIPT_ENTRY_REGEX);
    var matchedScriptSrcMatch = match.match(SCRIPT_SRC_REGEX);
    var matchedScriptSrc = matchedScriptSrcMatch && matchedScriptSrcMatch[2];

    if (entry && matchedScriptEntry) {
      throw new SyntaxError('You should not set multiply entry script!');
    } else {
      // append the domain while the script not have an protocol prefix
      if (matchedScriptSrc && !hasProtocol(matchedScriptSrc)) {
        matchedScriptSrc = domain + matchedScriptSrc;
      }

      entry = entry || matchedScriptEntry && matchedScriptSrc;
    }

    if (matchedScriptSrc) {
      scripts.push(matchedScriptSrc);
      return "<!-- script ".concat(matchedScriptSrc, " replaced -->");
    }

    return match;
  });
  scripts = scripts.filter(function (script) {
    // filter empty script
    return !!script;
  });
  return {
    template: template,
    scripts: scripts,
    styles: styles,
    // set the last script as entry if have not set
    entry: entry || scripts[scripts.length - 1]
  };
}